/*******************************************************************************//**
 * \file opx_tc.h
 * \brief Openvox Transcoding library for Vocallo/Openvox Transcoding Board
 * \brief Provides User a Unified/OS Agnostic API to Openvox Transcoding Board
 *
 * Author(s):   Bo Zhou <bo.zhou@openvox.cn>
 *
 * Copyright:	(c) 2010-2010 Bo Zhou <bo.zhou@openvox.cn>
*/

#ifndef __LIB_OPX_TC__
#define __LIB_OPX_TC__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <stdarg.h>
#include <pthread.h>
#include "opx_tc_version.h"


enum opxtc_codec_definition {
	OPXTC_CODEC_MIN_ID	= 0,

	OPXTC_CODEC_PCMU	= 1,
	OPXTC_CODEC_PCMA	= 2,
	OPXTC_CODEC_L16_1	= 3,
	OPXTC_CODEC_L16_2	= 4,
	OPXTC_CODEC_G729AB	= 5,
	OPXTC_CODEC_G722	= 6,
	OPXTC_CODEC_GSM_FR	= 7,
	OPXTC_CODEC_GSM_EFR	= 8,
	OPXTC_CODEC_AMR		= 9,
	OPXTC_CODEC_AMR_WB	= 10,
	OPXTC_CODEC_G726_16	= 11,
	OPXTC_CODEC_G726_24	= 12,
	OPXTC_CODEC_G726_32	= 13,
	OPXTC_CODEC_G726_40	= 14,
	OPXTC_CODEC_G723	= 15,
	OPXTC_CODEC_ILBC	= 16,
	OPXTC_CODEC_SPEEX	= 17,

	OPXTC_CODEC_MAX_ID, 
};

#define OPXTC_CODEC_ID_DECODE(id) \
		(id==OPXTC_CODEC_PCMU) ? "OPXTC_CODEC_PCMU" : \
		(id==OPXTC_CODEC_PCMA) ? "OPXTC_CODEC_PCMA" : \
		(id==OPXTC_CODEC_L16_1) ? "OPXTC_CODEC_L16_1" : \
		(id==OPXTC_CODEC_L16_2) ? "OPXTC_CODEC_L16_2" : \
		(id==OPXTC_CODEC_SPEEX) ? "OPXTC_CODEC_SPEEX" : \
		(id==OPXTC_CODEC_G729AB) ? "OPXTC_CODEC_G729AB" : \
		(id==OPXTC_CODEC_G722) ? "OPXTC_CODEC_G722" : \
		(id==OPXTC_CODEC_GSM_FR) ? "OPXTC_CODEC_GSM_FR" : \
		(id==OPXTC_CODEC_GSM_EFR) ? "OPXTC_CODEC_GSM_EFR" : \
		(id==OPXTC_CODEC_AMR) ? "OPXTC_CODEC_AMR" : \
		(id==OPXTC_CODEC_G723) ? "OPXTC_CODEC_G723" : \
		(id==OPXTC_CODEC_G726_32) ? "OPXTC_CODEC_G726_32" : \
		(id==OPXTC_CODEC_ILBC) ? "OPXTC_CODEC_ILIBC" : "Unknown"




enum opxtc_rpt_session_type
{
	OPXTC_RTP_SESSION_A,
	OPXTC_RTP_SESSION_B
};

struct opxtc_codec_reply;



typedef struct opxtc_codec_cfg
{
	uint32_t 	codec_id;
	uint32_t	ms;

	uint32_t	host_ip;
	uint32_t	host_netmask;
	uint32_t	host_udp_port;

} opxtc_codec_cfg_t, opxtc_codec_request_leg_t;

typedef struct  opxtc_codec_net_cfg
{
	uint32_t	codec_ip;
	uint32_t	codec_netmask;
	uint32_t	codec_udp_port;

	uint32_t	host_ip;
	uint32_t	host_netmask;
	uint32_t	host_udp_port;

	uint32_t	iana_code;

}opxtc_codec_net_cfg_t, opxtc_codec_reply_leg_t;


typedef struct opxtc_codec_reply
{
	uint16_t	codec_rtp_ctx_idx;

	opxtc_codec_reply_leg_t a;
	opxtc_codec_reply_leg_t b;

	void		*tx_fd;
	uint32_t	tx_iana;

	void		*rx_fd;
	uint32_t	rx_iana;

} opxtc_codec_reply_t;
 

typedef struct opxtc_codec_request
{
	void *usr_priv;

	uint8_t		rtcp_enable;

	opxtc_codec_request_leg_t a;
	opxtc_codec_request_leg_t b;

} opxtc_codec_request_t;

typedef struct opxtc_license_info
{
	uint32_t max;
	uint32_t current;
} opxtc_license_info;

#define OPXTC_MAX_MAC_RANGE 		32
#define OPXTC_MAX_HOST_VOCALLO_NIC 	32
#define OPXTC_MAX_ETH_SZ			6

typedef struct opxtc_eth_mac
{
	char mac_str[18];
	uint8_t mac[6];
}opxtc_eth_mac_t;

typedef struct opxtc_ip
{
	char ip_str[25];
	uint32_t ip;
}opxtc_ip_t;


typedef struct opxtc_vocallo_map {
	uint8_t init;

	opxtc_eth_mac_t v_mac;
	opxtc_ip_t 		v_ip;
	uint32_t		v_eth_port;

	uint32_t		tdm_timestamp_reg;
	uint32_t		error;
}opxtc_vocallo_map_t;

#define MAX_IP_STR_SIZE 100
typedef struct opxtc_nic_vocallo_map{
	uint8_t init;
	opxtc_eth_mac_t nic_mac;
	opxtc_ip_t nic_ip;
	opxtc_ip_t nic_netmask;

	opxtc_ip_t voc_base_ip;
	opxtc_ip_t voc_base_netmask;
	int        voc_base_udp_prot;

	int voc_mods_sz;
	opxtc_vocallo_map_t voc_map[OPXTC_MAX_MAC_RANGE];
}opxtc_nic_vocallo_map_t;


/*!
  \struct opxtc_host_nic_vocallo_cfg
  \brief Openvox transcode init structure used to configure the transcode with initial parmeters.
  \typedef opxtc_host_vocallo_cfg
  \brief Openvox transcode init structure used to configure the transcode with initial parmeters.
*/
typedef struct opxtc_host_nic_vocallo_cfg
{
	uint32_t 	vocallo_base_udp_port;
	uint32_t	vocallo_ip;
	uint32_t	vocallo_ip_netmask;

	uint32_t	host_ip;
	uint32_t	host_ip_netmask;

	opxtc_eth_mac_t	host_mac;

	/* Optional to select a speicifc vocallo */
	opxtc_eth_mac_t	vocallo_macs[OPXTC_MAX_MAC_RANGE];
	uint32_t		vocallo_macs_sz;

	uint8_t		silence_suppression;

} opxtc_host_nic_vocallo_cfg_t;


/*!
  \struct opxtc_init_cfg
  \brief Openvox transcode init structure used to configure the transcode with initial parmeters.
  \typedef opxtc_init_cfg_t
  \brief Openvox transcode init structure used to configure the transcode with initial parmeters.
*/ 

typedef struct opxtc_init_cfg
{
	uint32_t						host_nic_vocallo_sz;
	opxtc_host_nic_vocallo_cfg_t	host_nic_vocallo_cfg[OPXTC_MAX_HOST_VOCALLO_NIC];
	
	int (*log)( int level, char *fmt, ...);
	int (*create_rtp)(void *usr_priv, opxtc_codec_request_leg_t *codec_reg_leg, opxtc_codec_reply_leg_t* codec_reply_leg, void **rtp_fd);
	int (*destroy_rtp)(void *usr_priv, void *fd);

} opxtc_init_cfg_t;



/***************************
 Statistics
***************************/
/*!
	\struct opxtc_rtcp_sender_stats
	\brief RTCP Sender Statistics
	\typedef opxtc_rtcp_sender_stats_t
*/
typedef struct opxtc_rtcp_sender_stats
{
	uint32_t SentPktCnt[2];
	uint32_t SentByteCnt[2];
	uint32_t NtpTimestamp[2];
	uint32_t RtpTimestamp;
	uint32_t SenderSsrc;
	uint32_t SenderStartTime;
	uint32_t SenderReportCnt;
	uint32_t SenderState;
	uint32_t LastSenderReportTime;
	uint32_t SsrcChangeCnt;
} opxtc_rtcp_sender_stats_t;

/*!
	\struct opxtc_rtcp_receiver_stats
	\brief RTCP Receiver Statistics
	\typedef opxtc_rtcp_receiver_stats_t
*/
typedef struct opxtc_rtcp_receiver_stats
{
	uint32_t ulHighestSeqNumber;
	uint32_t ulCumulativeLostCnt;
	uint32_t ulFractionLost;
	uint32_t ulInterArrivalJitter;
	uint32_t ulMaxFractionLost;
	uint32_t ulMaxInterArrivalJitter;
	uint32_t ulAverageFractionLost;
	uint32_t ulAverageInterArrivalJitter;
	uint32_t ulReceiverStartTime;
	uint32_t ulReceiverReportCnt;
	uint32_t ulLastReceiverReportTime;
	uint32_t ulRemoteSenderSsrc;
	uint32_t ulAlarmRxThreshold;
	uint32_t ulAlarmRxThresholdCnt;
} opxtc_rtcp_receiver_stats_t;

/*!
	\struct opxtc_rtp_global_stats
	\brief RTP Global Statistics
	\typedef opxtc_rtp_global_stats_t
*/
typedef struct opxtc_rtp_global_stats
{
	uint32_t NumMember;
	uint32_t NumSender;
	uint32_t SessionStartTime;
	uint32_t RxByePktCnt;
	uint32_t RxSenderJoinCnt;
	uint32_t RxMemberJoinCnt;
	uint32_t ErrCollisionCnt;
	uint32_t ErrLoopCnt;
	uint32_t ErrRtcpPktInvalidCnt;
} opxtc_rtp_global_stats_t ;

/*!
	\struct opxtc_rtp_local_stats
	\brief Local RTP Statistics
	\typedef opxtc_rtp_local_stats_t
*/
typedef struct opxtc_rtp_local_stats
{
	/* Local Statistics */
	uint32_t ReceivedPktCnt[2];
	uint32_t ReceivedByteCnt[2];
	opxtc_rtcp_sender_stats_t SenderStats;
	opxtc_rtcp_receiver_stats_t ReceiverStats;
} opxtc_rtp_local_stats_t;


/*!
	\struct opxtc_rtp_remote_stats
	\brief Remote RTP Statistics
	\typedef opxtc_rtp_remote_stats_t
*/
typedef struct opxtc_rtp_remote_stats
{
	uint32_t RoundTripTime;
	uint32_t MaxRoundTripTime;
	uint32_t AverageRoundTripTime;
	opxtc_rtcp_sender_stats_t SenderStats;
	opxtc_rtcp_receiver_stats_t ReceiverStats;
} opxtc_rtp_remote_stats_t;

/*!
	\struct opxtc_rtp_stats
	\brief Full RTP Statistics both Local & Remote
	\typedef opxtc_rtp_stats_t
*/
typedef struct opxtc_rtp_stats
{
	opxtc_rtp_global_stats_t stats;
	opxtc_rtp_local_stats_t	 local_stats;
	opxtc_rtp_remote_stats_t remote_stats;

} opxtc_rtp_stats_t;

struct openvox_rtp_handle {
	void *ast_rtp; /* The asterisk RTP handle */
	unsigned int flags;
};

enum openvox_rtp_flags {
	OPX_RTP_TX = (1 << 0),
	OPX_RTP_RX = (1 << 1),
};

#define OPXTC_NIPV4(addr) \
	((unsigned char *)&addr)[3], \
	((unsigned char *)&addr)[2], \
	((unsigned char *)&addr)[1], \
	((unsigned char *)&addr)[0]

#define OPXTC_NIPV4_FMT "%u.%u.%u.%u"

/*!
  \enum OPXTC_LOGLEVEL_DEBUG
  \enum OPXTC_LOGLEVEL_WARN
  \enum OPXTC_LOGLEVEL_INFO
  \enum OPXTC_LOGLEVEL_STATS
  \enum OPXTC_LOGLEVEL_ERROR
  \enum OPXTC_LOGLEVEL_CRIT
*/
enum {
	OPXTC_LOGLEVEL_DEBUG=1,
	OPXTC_LOGLEVEL_WARN,
	OPXTC_LOGLEVEL_INFO,
	OPXTC_LOGLEVEL_STATS,
	OPXTC_LOGLEVEL_ERROR,
	OPXTC_LOGLEVEL_CRIT,
	OPXTC_LOGLEVEL_PUBLIC=100,
};

#define OPXTC_LOGLEVEL_DECODE(level) \
		((level==OPXTC_LOGLEVEL_DEBUG) ? "OPXTC_DEBUG" : \
		(level==OPXTC_LOGLEVEL_WARN) ? "OPXTC_WARN" : \
		(level==OPXTC_LOGLEVEL_INFO) ? "OPXTC_INFO" : \
		(level==OPXTC_LOGLEVEL_STATS) ? "OPXTC_STATS" : \
		(level==OPXTC_LOGLEVEL_ERROR) ? "OPXTC_ERROR" : \
		(level==OPXTC_LOGLEVEL_CRIT) ? "OPXTC_CRIT" : "Unkown")

int opxtc_get_nic_vocallo_map(opxtc_init_cfg_t *cfg, opxtc_nic_vocallo_map_t **nic_vocallo_map, int *nic_voc_map_idx);
int opxtc_detect_init_modules(opxtc_init_cfg_t *cfg, int *detected);
int opxtc_activate_modules(opxtc_init_cfg_t *cfg, int *activated);
int opxtc_deactivate_modules(void);

int opxtc_codec_ipv4_str_hex(const char *ip_str, uint32_t *ip_addr);
int opxtc_codec_ipv4_hex_to_str(uint32_t ip_addr, char *ip_str);
int opxtc_rtp_get_status(void *codec_obj, int rtp_a, opxtc_rtp_stats_t *rtp_stats);

int opxtc_create_transcoding_session (opxtc_codec_request_t *codec_req, opxtc_codec_reply_t *codec_reply);
int opxtc_destroy_transcoding_session (uint16_t ctx_idx);
int opxtc_all_free_sanity_check(void);
int opxtc_detected_vocallo_modules(void);

int opxtc_module_get_sw_version(void *module_obj);
int opxtc_module_get_license(void *module_obj);
void *opxtc_module_get_instance(int instance_idx);
int opxtc_module_reset(void *module_obj);
int opxtc_module_restore_session(void *module_obj);
int opxtc_module_create_host(void *module_obj);
int opxtc_module_write_file(void *module, char *local_file, char *remote_file, int progress);
char *opxtc_module_mac(void *module_obj);
int opxtc_set_iana_code_based_on_codec_id(int64_t codec_id, int64_t iana);
void opxtc_mac_addr_to_str(char *mac_str, uint8_t mac[OPXTC_MAX_ETH_SZ]);

int opxtc_module_read_sid(void *module_obj, char *local_file);

#endif
